/*
 MyemonTX Temperature and real power measurement example

 An example sketch for the emontx module for
 CT only electricity monitoring.

 Part of the openenergymonitor.org project
 Licence: GNU GPL V3

 Authors: Glyn Hudson, Trystan Lea
 Builds upon JeeLabs RF12 library and Arduino

THIS SKETCH REQUIRES:

Libraries in the standard arduino libraries folder:
- JeeLib		https://github.com/jcw/jeelib
- EmonLib		https://github.com/openenergymonitor/EmonLib.git
- OneWire library	http://www.pjrc.com/teensy/td_libs_OneWire.html
- DallasTemperature	http://download.milesburton.com/Arduino/MaximTemperature

Other files in project directory (should appear in the arduino tabs above)
- emontx_lib.ino
- print_to_serial.ino
emonTX node 10
OKG node 15
emonGLCD node 20
*/
//#define MYNODE 10                                                     // Should be unique on network, node ID 30 reserved for base station
#define freq RF12_433MHZ                                              // frequency - match to same frequency as RFM12B module (change to 868Mhz or 915Mhz if appropriate)
//#define group 210    
                                      // emonTx RFM12B wireless network group - needs to be same as emonBase and emonGLCD

const int UNO = 1;                                                    // Set to 0 if your not using the UNO bootloader (i.e using Duemilanove) - All Atmega's shipped from OpenEnergyMonitor come with Arduino Uno bootloader
#include <avr/wdt.h>

#include <JeeLib.h>                                                   // Download JeeLib: http://github.com/jcw/jeelib
ISR(WDT_vect)                                                         // Attached JeeLib sleep function to Atmega328 watchdog -enables MCU to be put into sleep mode inbetween readings to reduce power consumption
{
    Sleepy::watchdogEvent();
}

#include "EmonLib.h"

EnergyMonitor ct1;                                                    // Create  instances for each CT channel

#include <OneWire.h>
#include <DallasTemperature.h>
#include <RTClib.h>                                                   // Real time clock (RTC) - used for software RTC to reset kWh counters at midnight
#include <Wire.h>                                                     // Part of Arduino libraries - needed for RTClib
RTC_Millis RTC;
#define ONE_WIRE_BUS 4                                                // Data wire is plugged into port 2 on the Arduino
OneWire oneWire(ONE_WIRE_BUS);                                        // Setup a oneWire instance to communicate with any OneWire devices (not just Maxim/Dallas temperature ICs)
DallasTemperature sensors(&oneWire);                                  // Pass our oneWire reference to Dallas Temperature.

// By using direct addressing its possible to make sure that as you add temperature sensors
// the temperature sensor to variable mapping will not change.
// To find the addresses of your temperature sensors use the: **temperature_search sketch**
DeviceAddress address_T1 = { 0x28, 0xB2, 0x8F, 0xE0, 0x03, 0x00, 0x00, 0x7E };
DeviceAddress address_T2 = { 0x28, 0x85, 0x7A, 0xEE, 0x02, 0x00, 0x00, 0xDC };
DeviceAddress address_T3 = { 0x28, 0x95, 0x51, 0xEE, 0x02, 0x00, 0x00, 0x0F };
DeviceAddress address_T4 = { 0x28, 0x95, 0x51, 0xEE, 0x02, 0x00, 0x00, 0x0F };

typedef struct
{
    int Heaton;                                                       // Heaton
    int apparentPower;                                                // ApparentPower
    int T1;                                                           // temperature sensor 1
    int T2;                                                           // temperature sensor 2
    int T3;                                                           // temperature sensor 3
    int T4;      // temperature sensor 4
    int anything;
} Payload;
Payload emontx;

typedef struct
{
    int HeatonGLCD;
} PayloadGLCD;
PayloadGLCD emonglcd;

int threshold = 2800;   
int hour = 0, minute = 0;
int hourlast = 0, minutelast = 0;
int hourtotal = 0, minutetotal = 0;
int heating = 0, hourpast = 0;
int last_hour = 0;                                              // temp * 100
const int LEDpin = 9;

unsigned long last_emontx;                                            // Used to count time from last emontx update
unsigned long last_emonbase;                                          // Used to count time from last emontx update
boolean last_switch_state, switch_state;
unsigned long fast_update, slow_update;

void setup()
{
    Serial.begin(9600);
    rf12_initialize(10, freq,210);   // initialize RF
    Serial.println("emonTX Temperature power example");
    Serial.println("OpenEnergyMonitor.org");

    ct1.voltageTX(234.26, 1.7);                                       // Voltage: calibration, phase_shift
    ct1.currentTX(1, 111.1);                                          // Setup emonTX CT channel (channel, calibration)
// CT Calibration factor = CT ratio / burden resistance
// CT Calibration factor = (100A / 0.05A) x 18 Ohms
    sensors.begin();

 
                                
  //  rf12_sleep(RF12_SLEEP); // what does this do

    pinMode(LEDpin, OUTPUT);
    digitalWrite(LEDpin, HIGH);

    if (UNO) wdt_enable(WDTO_8S);
}

void loop()
{
       if (rf12_recvDone())
    {
        if (rf12_crc == 0 && (rf12_hdr & RF12_HDR_CTL) == 0)          // and no rf errors
        {
            int node_id = (rf12_hdr & 0x1F);
            if (node_id == 15)                                        // gets time from OKG - Where does OKG get time ( internet )
            {
                RTC.adjust(DateTime(2012, 1, 1, rf12_data[1], rf12_data[2], rf12_data[3]));
                last_emonbase = millis();
                Serial.println(" in rtc adjust");
            }
        }
    }
    
     if ((millis()-fast_update)>200)
    {
        fast_update = millis();

        DateTime now = RTC.now();
        last_hour = hour;
        hour = now.hour();
        minute = now.minute();

        if (last_hour == 23 && hour == 00)
        {
            updateusage ();

        }
    }
    
    sensors.requestTemperatures();                                    // Send the command to get temperatures

    emontx.T1 = sensors.getTempC(address_T1) * 100;
    emontx.T2 = sensors.getTempC(address_T2) * 100;
    emontx.T3 = sensors.getTempC(address_T3) * 100;
    emontx.T4 = sensors.getTempC(address_T4) * 100;
    emontx.anything = 24;
    ct1.calcVI(20,2000);                                              // Calculate all. No.of crossings, time-out

    if (emontx.T1 > threshold)

    {
        emontx.Heaton = 1;
    }
    else
    {
        emontx.Heaton = 0;
    }

    emontx.apparentPower = ct1.apparentPower;

    //print_to_serial();

    send_rf_data();                                                   // *SEND RF DATA* - see emontx_lib
    emontx_sleep(10);                                                 // sleep or delay in seconds - see emontx_lib
    digitalWrite(LEDpin, HIGH); delay(2); digitalWrite(LEDpin, LOW);  // flash LED
}
void updateusage ()
{
    heating = 0;
    if (last_hour == 23 && hour == 00)
    {
// something maybe add the hours to hour ? hour was 17 now 00 make it 17 + 24-17 + 01
    }
    hourpast = hour-hourlast;
    if (hourpast>0)
    {
        minutetotal = minutetotal + (60-minutelast) + minute;
// if hourpast is 2 or gretor add in 60 for each additional hour
    }
    else
    {
        minutetotal = minutetotal + (minute-minutelast);
    }
    Serial.print(" minutetotal  in loop ");
    Serial.println(minutetotal);
}
